@@ -32,7 +32,6 @@ import struct
 #
 import struct
 import StringIO
-import mmap
 import random
 
 NO_SIGNATURE = 'ff'
@@ -55,16 +54,6 @@ class BCDataStream(object):
         else:
             self.input += bytes
 
-    def map_file(self, file, start):  # Initialize with bytes from file
-        self.input = mmap.mmap(file.fileno(), 0, access=mmap.ACCESS_READ)
-        self.read_cursor = start
-
-    def seek_file(self, position):
-        self.read_cursor = position
-
-    def close_file(self):
-        self.input.close()
-
     def read_string(self):
         # Strings are encoded depending on length:
         # 0 to 252 :  1-byte-length followed by bytes (if any)
@@ -481,7 +470,7 @@ class Transaction:
         return self.raw
 
     def __init__(self, raw):
-        self.raw = raw
+        self.raw = raw.strip() if raw else None
         self.inputs = None
 
     def update(self, raw):
@@ -541,10 +530,11 @@ class Transaction:
     @classmethod
     def sweep(klass, privkeys, network, to_address, fee):
         inputs = []
+        keypairs = {}
         for privkey in privkeys:
             pubkey = public_key_from_private_key(privkey)
             address = address_from_private_key(privkey)
-            u = network.synchronous_get([ ('blockchain.address.listunspent',[address])])[0]
+            u = network.synchronous_get(('blockchain.address.listunspent',[address]))
             pay_script = klass.pay_script('address', address)
             for item in u:
                 item['scriptPubKey'] = pay_script
@@ -557,6 +547,7 @@ class Transaction:
                 item['signatures'] = [None]
                 item['num_sig'] = 1
             inputs += u
+            keypairs[pubkey] = privkey
 
         if not inputs:
             return
@@ -564,7 +555,7 @@ class Transaction:
         total = sum(i.get('value') for i in inputs) - fee
         outputs = [('address', to_address, total)]
         self = klass.from_io(inputs, outputs)
-        self.sign({ pubkey:privkey })
+        self.sign(keypairs)
         return self
 
     @classmethod
