@@ -16,20 +16,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-import sys
 import os
 import hashlib
 import ast
 import threading
 import random
 import time
-import math
 import json
 import copy
 from operator import itemgetter
 
-from util import print_msg, print_error, NotEnoughFunds
-from util import profiler
+from util import NotEnoughFunds, PrintError, profiler
 
 from bitcoin import *
 from account import *
@@ -38,25 +35,24 @@ from version import *
 from transaction import Transaction
 from plugins import run_hook
 import bitcoin
-from synchronizer import WalletSynchronizer
+from synchronizer import Synchronizer
 from mnemonic import Mnemonic
 
 import paymentrequest
 
-
-
 # internal ID for imported account
 IMPORTED_ACCOUNT = '/x'
 
 
-class WalletStorage(object):
+class WalletStorage(PrintError):
 
     def __init__(self, path):
         self.lock = threading.RLock()
         self.data = {}
         self.path = path
         self.file_exists = False
-        print_error( "wallet path", self.path )
+        self.modified = False
+        self.print_error("wallet path", self.path)
         if self.path:
             self.read(self.path)
 
@@ -87,7 +83,7 @@ class WalletStorage(object):
                     json.dumps(key)
                     json.dumps(value)
                 except:
-                    print_error('Failed to convert label to json format', key)
+                    self.print_error('Failed to convert label to json format', key)
                     continue
                 self.data[key] = value
         self.file_exists = True
@@ -99,27 +95,32 @@ class WalletStorage(object):
                 v = default
             else:
                 v = copy.deepcopy(v)
-            return v
+        return v
 
     def put(self, key, value, save = True):
         try:
             json.dumps(key)
             json.dumps(value)
         except:
-            print_error("json error: cannot save", key)
+            self.print_error("json error: cannot save", key)
             return
         with self.lock:
             if value is not None:
-                self.data[key] = copy.deepcopy(value)
+                if self.data.get(key) != value:
+                    self.modified = True
+                    self.data[key] = copy.deepcopy(value)
             elif key in self.data:
+                self.modified = True
                 self.data.pop(key)
             if save:
                 self.write()
 
     def write(self):
         assert not threading.currentThread().isDaemon()
-        temp_path = "%s.tmp.%s" % (self.path, os.getpid())
+        if not self.modified:
+            return
         s = json.dumps(self.data, indent=4, sort_keys=True)
+        temp_path = "%s.tmp.%s" % (self.path, os.getpid())
         with open(temp_path, "w") as f:
             f.write(s)
             f.flush()
@@ -133,10 +134,11 @@ class WalletStorage(object):
         if 'ANDROID_DATA' not in os.environ:
             import stat
             os.chmod(self.path,stat.S_IREAD | stat.S_IWRITE)
+        self.print_error("saved")
 
 
 
-class Abstract_Wallet(object):
+class Abstract_Wallet(PrintError):
     """
     Wallet classes are created to handle various address generation methods.
     Completion states (watching-only, single account, no seed, etc) are handled inside classes.
@@ -171,7 +173,8 @@ class Abstract_Wallet(object):
 
         # spv
         self.verifier = None
-        # Transactions pending verification.  Each value is the transaction height.  Access with self.lock.
+        # Transactions pending verification.  A map from tx hash to transaction
+        # height.  Access is not contended so no lock is needed.
         self.unverified_tx = {}
         # Verified transactions.  Each value is a (height, timestamp, block_pos) tuple.  Access with self.lock.
         self.verified_tx   = storage.get('verified_tx3',{})
@@ -190,6 +193,9 @@ class Abstract_Wallet(object):
         if self.storage.get('wallet_type') is None:
             self.storage.put('wallet_type', self.wallet_type, True)
 
+    def diagnostic_name(self):
+        return self.basename()
+
     @profiler
     def load_transactions(self):
         self.txi = self.storage.get('txi', {})
@@ -201,7 +207,7 @@ class Abstract_Wallet(object):
             tx = Transaction(raw)
             self.transactions[tx_hash] = tx
             if self.txi.get(tx_hash) is None and self.txo.get(tx_hash) is None and (tx_hash not in self.pruned_txo.values()):
-                print_error("removing unreferenced tx", tx_hash)
+                self.print_error("removing unreferenced tx", tx_hash)
                 self.transactions.pop(tx_hash)
 
     @profiler
@@ -290,7 +296,7 @@ class Abstract_Wallet(object):
                 except:
                     pass
             else:
-                print_error("cannot load account", v)
+                self.print_error("cannot load account", v)
 
     def synchronize(self):
         pass
@@ -364,9 +370,9 @@ class Abstract_Wallet(object):
                 changed = True
 
         if changed:
+            run_hook('set_label', self, name, text)
             self.storage.put('labels', self.labels, True)
 
-        run_hook('set_label', name, text, changed)
         return changed
 
     def addresses(self, include_change = True):
@@ -416,11 +422,13 @@ class Abstract_Wallet(object):
         return decrypted
 
     def add_unverified_tx(self, tx_hash, tx_height):
-        if tx_height > 0:
-            with self.lock:
-                self.unverified_tx[tx_hash] = tx_height
+        # Only add if confirmed and not verified
+        if tx_height > 0 and tx_hash not in self.verified_tx:
+            self.unverified_tx[tx_hash] = tx_height
 
     def add_verified_tx(self, tx_hash, info):
+        # Remove from the unverified map and add to the verified map and
+        self.unverified_tx.pop(tx_hash, None)
         with self.lock:
             self.verified_tx[tx_hash] = info  # (tx_height, timestamp, pos)
         self.storage.put('verified_tx3', self.verified_tx, True)
@@ -429,14 +437,8 @@ class Abstract_Wallet(object):
         self.network.trigger_callback('verified', (tx_hash, conf, timestamp))
 
     def get_unverified_txs(self):
-        '''Returns a list of tuples (tx_hash, height) that are unverified and not beyond local height'''
-        txs = []
-        with self.lock:
-            for tx_hash, tx_height in self.unverified_tx.items():
-                # do not request merkle branch before headers are available
-                if tx_hash not in self.verified_tx and tx_height <= self.get_local_height():
-                    txs.append((tx_hash, tx_height))
-        return txs
+        '''Returns a map from tx hash to transaction height'''
+        return self.unverified_tx
 
     def undo_verifications(self, height):
         '''Used by the verifier when a reorg has happened'''
@@ -473,7 +475,7 @@ class Abstract_Wallet(object):
         "return position, even if the tx is unverified"
         with self.lock:
             x = self.verified_tx.get(tx_hash)
-            y = self.unverified_tx.get(tx_hash)
+        y = self.unverified_tx.get(tx_hash)
         if x:
             height, timestamp, pos = x
             return height, pos
@@ -692,7 +694,7 @@ class Abstract_Wallet(object):
         for addr, l in dd.items():
             for n, v, is_cb in l:
                 if n == prevout_n:
-                    print_error("found pay-to-pubkey address:", addr)
+                    self.print_error("found pay-to-pubkey address:", addr)
                     return addr
 
     def add_transaction(self, tx_hash, tx):
@@ -748,7 +750,7 @@ class Abstract_Wallet(object):
 
     def remove_transaction(self, tx_hash):
         with self.transaction_lock:
-            print_error("removing tx from history", tx_hash)
+            self.print_error("removing tx from history", tx_hash)
             #tx = self.transactions.pop(tx_hash)
             for ser, hh in self.pruned_txo.items():
                 if hh == tx_hash:
@@ -844,7 +846,7 @@ class Abstract_Wallet(object):
 
         # fixme: this may happen if history is incomplete
         if balance not in [None, 0]:
-            print_error("Error: history not synchronized")
+            self.print_error("Error: history not synchronized")
             return []
 
         return h2
@@ -933,7 +935,7 @@ class Abstract_Wallet(object):
                     fee = fixed_fee if fixed_fee is not None else self.estimated_fee(tx, fee_per_kb)
                     continue
                 break
-        print_error("using %d inputs"%len(tx.inputs))
+        self.print_error("using %d inputs"%len(tx.inputs))
 
         # change address
         if not change_addr:
@@ -966,16 +968,16 @@ class Abstract_Wallet(object):
             change_amount = total - ( amount + fee )
             if change_amount > DUST_THRESHOLD:
                 tx.outputs.append(('address', change_addr, change_amount))
-                print_error('change', change_amount)
+                self.print_error('change', change_amount)
             else:
-                print_error('not keeping dust', change_amount)
+                self.print_error('not keeping dust', change_amount)
         else:
-            print_error('not keeping dust', change_amount)
+            self.print_error('not keeping dust', change_amount)
 
         # Sort the inputs and outputs deterministically
         tx.BIP_LI01_sort()
 
-        run_hook('make_unsigned_transaction', tx)
+        run_hook('make_unsigned_transaction', self, tx)
         return tx
 
     def mktx(self, outputs, password, config, fee=None, change_addr=None, domain=None):
@@ -1023,7 +1025,12 @@ class Abstract_Wallet(object):
         # Sign
         if keypairs:
             tx.sign(keypairs)
-        run_hook('sign_transaction', tx, password)
+        # Run hook, and raise if error
+        tx.error = None
+        run_hook('sign_transaction', self, tx, password)
+        if tx.error:
+            raise BaseException(tx.error)
+
 
     def sendtx(self, tx):
         # synchronous
@@ -1086,9 +1093,7 @@ class Abstract_Wallet(object):
             return True
         return False
 
-    def set_verifier(self, verifier):
-        self.verifier = verifier
-
+    def prepare_for_verifier(self):
         # review transactions that are in the history
         for addr, hist in self.history.items():
             for tx_hash, tx_height in hist:
@@ -1100,27 +1105,26 @@ class Abstract_Wallet(object):
             vr = self.verified_tx.keys() + self.unverified_tx.keys()
         for tx_hash in self.transactions.keys():
             if tx_hash not in vr:
-                print_error("removing transaction", tx_hash)
+                self.print_error("removing transaction", tx_hash)
                 self.transactions.pop(tx_hash)
 
     def start_threads(self, network):
         from verifier import SPV
         self.network = network
         if self.network is not None:
+            self.prepare_for_verifier()
             self.verifier = SPV(self.network, self)
-            self.verifier.start()
-            self.set_verifier(self.verifier)
-            self.synchronizer = WalletSynchronizer(self, network)
-            network.jobs.append(self.synchronizer.main_loop)
+            self.synchronizer = Synchronizer(self, network)
+            network.add_jobs([self.verifier, self.synchronizer])
         else:
             self.verifier = None
             self.synchronizer = None
 
     def stop_threads(self):
         if self.network:
-            self.verifier.stop()
-            self.network.jobs.remove(self.synchronizer.main_loop)
+            self.network.remove_jobs([self.synchronizer, self.verifier])
             self.synchronizer = None
+            self.verifier = None
             self.storage.put('stored_height', self.get_local_height(), True)
 
     def restore(self, cb):
@@ -1744,19 +1748,19 @@ class BIP32_HD_Wallet(BIP32_Wallet):
                 self.next_account = self.get_next_account(None)
                 self.storage.put('next_account2', self.next_account)
             except:
-                print_error('cannot get next account')
+                self.print_error('cannot get next account')
         # check pending account
         if self.next_account is not None:
             next_id, next_xpub, next_pubkey, next_address = self.next_account
             if self.address_is_old(next_address):
-                print_error("creating account", next_id)
+                self.print_error("creating account", next_id)
                 self.add_account(next_id, BIP32_Account({'xpub':next_xpub}))
                 # here the user should get a notification
                 self.next_account = None
                 self.storage.put('next_account2', self.next_account)
             elif self.history.get(next_address, []):
                 if next_id not in self.accounts:
-                    print_error("create pending account", next_id)
+                    self.print_error("create pending account", next_id)
                     self.accounts[next_id] = PendingAccount({'pending':True, 'address':next_address, 'pubkey':next_pubkey})
                     self.save_accounts()
 
@@ -1924,9 +1928,12 @@ class Wallet(object):
 
         wallet_type = storage.get('wallet_type')
         if wallet_type:
-            for cat, t, name, c in wallet_types:
+            for cat, t, name, loader in wallet_types:
                 if t == wallet_type:
-                    WalletClass = c
+                    if cat in ['hardware', 'twofactor']:
+                        WalletClass = lambda storage: apply(loader().constructor, (storage,))
+                    else:
+                        WalletClass = loader
                     break
             else:
                 if re.match('(\d+)of(\d+)', wallet_type):
